@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2014 Dovecot authors, see the included COPYING file */
 
 #include "pop3-common.h"
 #include "array.h"
@@ -237,6 +237,7 @@ bool client_update_mails(struct client *client)
 	struct mail_search_args *search_args;
 	struct mail_search_context *ctx;
 	struct mail *mail;
+	ARRAY_TYPE(seq_range) deleted_msgs, seen_msgs;
 	uint32_t msgnum, bit;
 	bool ret = TRUE;
 
@@ -245,27 +246,35 @@ bool client_update_mails(struct client *client)
 		return TRUE;
 	}
 
+	/* translate msgnums to sequences (in case POP3 ordering is
+	   different) */
+	t_array_init(&deleted_msgs, 8);
+	if (client->deleted_bitmask != NULL) {
+		for (msgnum = 0; msgnum < client->messages_count; msgnum++) {
+			bit = 1 << (msgnum % CHAR_BIT);
+			if ((client->deleted_bitmask[msgnum / CHAR_BIT] & bit) != 0)
+				seq_range_array_add(&deleted_msgs, client->msgnum_to_seq_map[msgnum]);
+		}
+	}
+	t_array_init(&seen_msgs, 8);
+	if (client->seen_bitmask != NULL) {
+		for (msgnum = 0; msgnum < client->messages_count; msgnum++) {
+			bit = 1 << (msgnum % CHAR_BIT);
+			if ((client->seen_bitmask[msgnum / CHAR_BIT] & bit) != 0)
+				seq_range_array_add(&seen_msgs, client->msgnum_to_seq_map[msgnum]);
+		}
+	}
+
 	search_args = pop3_search_build(client, 0);
-	ctx = mailbox_search_init(client->trans, search_args,
-				  pop3_sort_program, 0, NULL);
+	ctx = mailbox_search_init(client->trans, search_args, NULL, 0, NULL);
 	mail_search_args_unref(&search_args);
 
 	msgnum = 0;
 	while (mailbox_search_next(ctx, &mail)) {
-		if (client_verify_ordering(client, mail, msgnum) < 0) {
-			ret = FALSE;
-			break;
-		}
-
-		bit = 1 << (msgnum % CHAR_BIT);
-		if (client->deleted_bitmask != NULL &&
-		    (client->deleted_bitmask[msgnum / CHAR_BIT] & bit) != 0) {
+		if (seq_range_exists(&deleted_msgs, mail->seq))
 			client_expunge(client, mail);
-		} else if (client->seen_bitmask != NULL &&
-			   (client->seen_bitmask[msgnum / CHAR_BIT] & bit) != 0) {
+		else if (seq_range_exists(&seen_msgs, mail->seq))
 			mail_update_flags(mail, MODIFY_ADD, MAIL_SEEN);
-		}
-		msgnum++;
 	}
 
 	client->seen_change_count = 0;
@@ -558,7 +567,7 @@ struct cmd_uidl_context {
 	bool list_all;
 };
 
-static void
+static int
 pop3_get_uid(struct client *client, struct mail *mail, string_t *str,
 	     bool *permanent_uidl_r)
 {
@@ -579,7 +588,7 @@ pop3_get_uid(struct client *client, struct mail *mail, string_t *str,
 		str_append(str, uidl);
 		/* UIDL is already permanent */
 		*permanent_uidl_r = TRUE;
-		return;
+		return 0;
 	}
 
 	*permanent_uidl_r = FALSE;
@@ -587,7 +596,7 @@ pop3_get_uid(struct client *client, struct mail *mail, string_t *str,
 	if (client->set->pop3_reuse_xuidl &&
 	    mail_get_first_header(mail, "X-UIDL", &uidl) > 0) {
 		str_append(str, uidl);
-		return;
+		return 0;
 	}
 
 	tab = t_malloc(sizeof(static_tab));
@@ -595,39 +604,48 @@ pop3_get_uid(struct client *client, struct mail *mail, string_t *str,
 	tab[0].value = t_strdup_printf("%u", client->uid_validity);
 
 	if ((client->uidl_keymask & UIDL_UID) != 0) {
-		if (i_snprintf(uid_str, sizeof(uid_str), "%u",
-			       mail->uid) < 0)
+		if (i_snprintf(uid_str, sizeof(uid_str), "%u", mail->uid) < 0)
 			i_unreached();
 		tab[1].value = uid_str;
 	}
 	if ((client->uidl_keymask & UIDL_MD5) != 0) {
 		if (mail_get_special(mail, MAIL_FETCH_HEADER_MD5,
-				     &tab[2].value) < 0 ||
-		    *tab[2].value == '\0') {
-			/* broken */
-			i_fatal("UIDL: Header MD5 not found "
+				     &tab[2].value) < 0) {
+			i_error("UIDL: Header MD5 lookup failed: %s",
+				mailbox_get_last_error(mail->box, NULL));
+			return -1;
+		} else if (*tab[2].value == '\0') {
+			i_error("UIDL: Header MD5 not found "
 				"(pop3_uidl_format=%%m not supported by storage?)");
+			return -1;
 		}
 	}
 	if ((client->uidl_keymask & UIDL_FILE_NAME) != 0) {
 		if (mail_get_special(mail, MAIL_FETCH_UIDL_FILE_NAME,
-				     &tab[3].value) < 0 ||
-		    *tab[3].value == '\0') {
-			/* broken */
-			i_fatal("UIDL: File name not found "
+				     &tab[3].value) < 0) {
+			i_error("UIDL: File name lookup failed: %s",
+				mailbox_get_last_error(mail->box, NULL));
+			return -1;
+		} else if (*tab[3].value == '\0') {
+			i_error("UIDL: File name not found "
 				"(pop3_uidl_format=%%f not supported by storage?)");
+			return -1;
 		}
 	}
 	if ((client->uidl_keymask & UIDL_GUID) != 0) {
 		if (mail_get_special(mail, MAIL_FETCH_GUID,
-				     &tab[4].value) < 0 ||
-		    *tab[4].value == '\0') {
-			/* broken */
-			i_fatal("UIDL: Message GUID not found "
+				     &tab[4].value) < 0) {
+			i_error("UIDL: Message GUID lookup failed: %s",
+				mailbox_get_last_error(mail->box, NULL));
+			return -1;
+		} else if (*tab[4].value == '\0') {
+			i_error("UIDL: Message GUID not found "
 				"(pop3_uidl_format=%%g not supported by storage?)");
+			return -1;
 		}
 	}
 	var_expand(str, client->mail_set->pop3_uidl_format, tab);
+	return 0;
 }
 
 static bool
@@ -669,6 +687,7 @@ static bool list_uids_iter(struct client *client, struct cmd_uidl_context *ctx)
 {
 	string_t *str;
 	bool permanent_uidl, found = FALSE;
+	bool failed = FALSE;
 
 	if (client->message_uidls != NULL)
 		return list_uidls_saved_iter(client, ctx);
@@ -677,8 +696,10 @@ static bool list_uids_iter(struct client *client, struct cmd_uidl_context *ctx)
 	while (mailbox_search_next(ctx->search_ctx, &ctx->mail)) {
 		uint32_t msgnum = ctx->msgnum++;
 
-		if (client_verify_ordering(client, ctx->mail, msgnum) < 0)
-			i_fatal("Can't finish POP3 UIDL command");
+		if (client_verify_ordering(client, ctx->mail, msgnum) < 0) {
+			failed = TRUE;
+			break;
+		}
 		if (client->deleted) {
 			if (client->deleted_bitmask[msgnum / CHAR_BIT] &
 			    (1 << (msgnum % CHAR_BIT)))
@@ -687,7 +708,10 @@ static bool list_uids_iter(struct client *client, struct cmd_uidl_context *ctx)
 		found = TRUE;
 
 		str_truncate(str, 0);
-		pop3_get_uid(client, ctx->mail, str, &permanent_uidl);
+		if (pop3_get_uid(client, ctx->mail, str, &permanent_uidl) < 0) {
+			failed = TRUE;
+			break;
+		}
 		if (client->set->pop3_save_uidl && !permanent_uidl)
 			mail_update_pop3_uidl(ctx->mail, str_c(str));
 
@@ -707,10 +731,12 @@ static bool list_uids_iter(struct client *client, struct cmd_uidl_context *ctx)
 
 	client->cmd = NULL;
 
-	if (ctx->list_all)
+	if (ctx->list_all && !failed)
 		client_send_line(client, ".");
 	i_free(ctx);
-	return found;
+	if (failed)
+		client_disconnect(client, "POP3 UIDLs couldn't be listed");
+	return found || failed;
 }
 
 static void cmd_uidl_callback(struct client *client)
@@ -745,11 +771,12 @@ static void client_uidls_save(struct client *client)
 	struct mail_search_args *search_args;
 	struct mail *mail;
 	HASH_TABLE_TYPE(uidl_counter) prev_uidls;
+	const char **seq_uidls;
 	string_t *str;
 	char *uidl;
 	enum mail_fetch_field wanted_fields;
 	uint32_t msgnum;
-	bool permanent_uidl, uidl_duplicates_rename;
+	bool permanent_uidl, uidl_duplicates_rename, failed = FALSE;
 
 	i_assert(client->message_uidls == NULL);
 
@@ -759,36 +786,49 @@ static void client_uidls_save(struct client *client)
 		wanted_fields |= MAIL_FETCH_HEADER_MD5;
 
 	search_ctx = mailbox_search_init(client->trans, search_args,
-					 pop3_sort_program,
-					 wanted_fields, NULL);
+					 NULL, wanted_fields, NULL);
 	mail_search_args_unref(&search_args);
 
 	uidl_duplicates_rename =
 		strcmp(client->set->pop3_uidl_duplicates, "rename") == 0;
 	hash_table_create(&prev_uidls, default_pool, 0, str_hash, strcmp);
 	client->uidl_pool = pool_alloconly_create("message uidls", 1024);
-	client->message_uidls = p_new(client->uidl_pool, const char *,
-				      client->messages_count+1);
 
-	str = t_str_new(128); msgnum = 0;
+	/* first read all the UIDLs into a temporary [seq] array */
+	seq_uidls = i_new(const char *, client->messages_count);
+	str = t_str_new(128);
 	while (mailbox_search_next(search_ctx, &mail)) {
-		if (client_verify_ordering(client, mail, msgnum) < 0)
-			i_fatal("Can't finish POP3 UIDL command");
-
 		str_truncate(str, 0);
-		pop3_get_uid(client, mail, str, &permanent_uidl);
-		if (client->set->pop3_save_uidl && !permanent_uidl)
-			mail_update_pop3_uidl(mail, str_c(str));
-
+		if (pop3_get_uid(client, mail, str, &permanent_uidl) < 0) {
+			failed = TRUE;
+			break;
+		}
 		if (uidl_duplicates_rename)
 			uidl_rename_duplicate(str, prev_uidls);
+
 		uidl = p_strdup(client->uidl_pool, str_c(str));
-		client->message_uidls[msgnum] = uidl;
+		if (client->set->pop3_save_uidl && !permanent_uidl)
+			mail_update_pop3_uidl(mail, uidl);
+
+		seq_uidls[mail->seq-1] = uidl;
 		hash_table_insert(prev_uidls, uidl, POINTER_CAST(1));
-		msgnum++;
 	}
 	(void)mailbox_search_deinit(&search_ctx);
 	hash_table_destroy(&prev_uidls);
+
+	if (failed) {
+		pool_unref(&client->uidl_pool);
+		i_free(seq_uidls);
+		return;
+	}
+	/* map UIDLs to msgnums (in case POP3 sort ordering is different) */
+	client->message_uidls = p_new(client->uidl_pool, const char *,
+				      client->messages_count+1);
+	for (msgnum = 0; msgnum < client->messages_count; msgnum++) {
+		client->message_uidls[msgnum] =
+			seq_uidls[client->msgnum_to_seq_map[msgnum]];
+	}
+	i_free(seq_uidls);
 }
 
 static struct cmd_uidl_context *
